# -*- coding:utf-8 -*-

from foxrelax.go.gotypes import Point

__all__ = ['is_point_an_eye']


def is_point_an_eye(board, point, color):
    # 眼必须是一个空点
    if board.get(point) is not None:
        return False

    # 所有相邻的点必须是己方棋子
    for neighbor in board.neighbors(point):
        neighbor_color = board.get(neighbor)
        if neighbor_color != color:
            return False

    # 如果这个空点位于棋盘内部, 己方棋子至少得控制4个对角相邻点中的3个; 如果空点在边缘,
    # 则必须控制所有的对角相邻点
    friendly_corners = 0
    off_board_corners = 0
    corners = [
        Point(point.row - 1, point.col - 1),
        Point(point.row - 1, point.col + 1),
        Point(point.row + 1, point.col - 1),
        Point(point.row + 1, point.col + 1),
    ]
    for corner in corners:
        if board.is_on_grid(corner):
            corner_color = board.get(corner)
            if corner_color == color:
                friendly_corners += 1
        else:
            off_board_corners += 1
    # 空点在边缘或者角落
    if off_board_corners > 0:
        return off_board_corners + friendly_corners == 4

    # 空点在棋盘内部
    return friendly_corners >= 3